<?php

/*

    ユーティリティ関数

*/


// ロック用クラス
class Lock
{
    private $dir;

    public function __construct( $dir ){

        $this->dir = $dir;

        $lock = get_vardir().$this->dir;
        $timeout = LOCKTIMEOUT;
        while( ! @mkdir( $lock ) ){

            if( ! $timeout ){
                rmdir( $lock ); 
                break;
            }
            else{
                sleep( 1 );
                --$timeout;
            }
        }

        show_debug( 'lock: '.$lock );
    }


    function __destruct()
    {
        $lock = get_vardir().$this->dir;
        if( file_exists( $lock ) ) rmdir( $lock );

        show_debug( 'unlock: '.$lock );
    }

}


// デフォルトステータス値
function get_default_status()
{
    return array( 
                  0, // ID
                  0, // PID
                  0, // CHANNEL
                  0, // STARTTIME
                  0, // DURATION
                  '', // OUTDIR
                  '', // NAMEBASE
                  0, // UDP
                  '', // IP
                  0, // RESERV
                  0 // PORT
        );
}


// デフォルト録画設定値
function get_default_setting()
{
    return array(
                  0, // SET_ID
                  '', // SET_COMMENT
                  0, // SET_CHANNEL

                  0, // SET_YEAR
                  0, // SET_MONTH
                  0, // SET_DAY
                  0, // SET_HOUR
                  0, // SET_MIN

                  0, // SET_SUN
                  0, // SET_MON
                  0, // SET_TUE
                  0, // SET_WED
                  0, // SET_THU
                  0, // SET_FRI
                  0, // SET_SAT

                  3600, // SET_DURATION
                  SPOOL_DIRECTORY, // SET_OUTDIR
                  '', // SET_NAMEBASE
                  0, // SET_OVERWRITE
                  0, // SET_UDP
                  '', // SET_IP
                  1234, // SET_PORT

                  1, // SET_ACTIVE
                  MODE_ONCE, // SET_MODE
                  0, // SET_SLEEPSEC
                  0, // SET_SUSPEND

                  0, // SET_RES1
                  0, // SET_RES2
                  0  // SET_RES3
        );
}


// ホームディレクトリ取得
function get_homedir()
{
    $home = HOME;
    if( $home[ strlen( $home ) -1 ] != '/' ) $home = $home.'/';
    return $home;
}


// 作業ディレクトリパス取得
function get_vardir()
{
    return get_homedir().VARDIR.'/';
}


// 設定ファイル名取得
function get_settingfile()
{
    return get_vardir().SETTINGFILE;
}


// 予約ファイル名取得
function get_reservfile()
{
    return get_vardir().RESERVFILE;
}


// ステータスファイル名取得
function get_statusfile()
{
    return get_vardir().STATUSFILE;
}


// $setttingfile を読み込んで ID から録画情報を取得
function get_setting_by_id( $id, $settingfile )
{
    $setting_array = array();
    if( file_exists( $settingfile ) ) read_csv( $settingfile, $setting_array );

    foreach( $setting_array as $setting ){

        if( $setting[ SET_ID ] == $id ) return $setting;
    }

    return FALSE;
}


// POST 時に $setting 番の録画設定項目を更新する
function update_settingitem( $settingno, &$setting )
{
    global $CHANNEL_TABLE;

    show_debug( 'update_settingitem settingno = '.$settingno );

    // アクティブ
    if( isset( $_POST[ 'active'.$settingno ] ) ) $setting[ SET_ACTIVE ] = 0;
    else $setting[ SET_ACTIVE ] = 1;

    // コメント
    if( isset( $_POST[ 'comment'.$settingno ] ) ) $setting[ SET_COMMENT ] = trim( $_POST[ 'comment'.$settingno ] );

    // チャンネル
    if( isset( $_POST[ 'channel'.$settingno ] ) ){

        $channel = -1;
        foreach( $CHANNEL_TABLE as $ch => $chname ){

            if( $_POST[ 'channel'.$settingno ] == $chname.'('.$ch.')' ){
                $channel = $ch;
                break;
            }
        }

        $setting[ SET_CHANNEL ] = $channel;
    }

    // 録画のモード
    if( isset( $_POST[ 'type'.$settingno ] ) ){
        if( $_POST[ 'type'.$settingno ] == 'once' ) $setting[ SET_MODE ] = MODE_ONCE;
        else $setting[ SET_MODE ] = MODE_WEEK;
    }

    // 日時
    if( $setting[ SET_MODE ] == MODE_ONCE ){
        if( isset( $_POST[ 'month'.$settingno ] ) ) $setting[ SET_MONTH ] = $_POST[ 'month'.$settingno ];
        if( isset( $_POST[ 'day'.$settingno ] ) ) $setting[ SET_DAY ] = $_POST[ 'day'.$settingno ];
    }

    // 曜日
    if( $setting[ SET_MODE ] == MODE_WEEK ){
        if( isset( $_POST[ 'sun'.$settingno ] ) ) $setting[ SET_SUN ] = 1; else $setting[ SET_SUN ] = 0;
        if( isset( $_POST[ 'mon'.$settingno ] ) ) $setting[ SET_MON ] = 1; else $setting[ SET_MON ] = 0;
        if( isset( $_POST[ 'tue'.$settingno ] ) ) $setting[ SET_TUE ] = 1; else $setting[ SET_TUE ] = 0;
        if( isset( $_POST[ 'wed'.$settingno ] ) ) $setting[ SET_WED ] = 1; else $setting[ SET_WED ] = 0;
        if( isset( $_POST[ 'thu'.$settingno ] ) ) $setting[ SET_THU ] = 1; else $setting[ SET_THU ] = 0;
        if( isset( $_POST[ 'fri'.$settingno ] ) ) $setting[ SET_FRI ] = 1; else $setting[ SET_FRI ] = 0;
        if( isset( $_POST[ 'sat'.$settingno ] ) ) $setting[ SET_SAT ] = 1; else $setting[ SET_SAT ] = 0;
    }

    // 時刻
    if( isset( $_POST[ 'hour'.$settingno ] ) ) $setting[ SET_HOUR ] = (int) $_POST[ 'hour'.$settingno ];    
    if( isset( $_POST[ 'min'.$settingno ] ) ) $setting[ SET_MIN ] = (int) $_POST[ 'min'.$settingno ];    

    // 録画時間
    if( isset( $_POST[ 'duration'.$settingno ] ) ) $setting[ SET_DURATION ] = (int)( $_POST[ 'duration'.$settingno ] )* 60;

    // スリープ時間
    if( isset( $_POST[ 'sleepsec'.$settingno ] ) ) $setting[ SET_SLEEPSEC ] = 60 - (int)$_POST[ 'sleepsec'.$settingno ];

    // ディレクトリ
    $setting[ SET_OUTDIR ] = trim( SPOOL_DIRECTORY );

    // ファイル名
    if( isset( $_POST[ 'namebase'.$settingno ] ) ) $setting[ SET_NAMEBASE ] = trim( $_POST[ 'namebase'.$settingno ] );
    if( isset( $_POST[ 'overwrite'.$settingno ] ) ) $setting[ SET_OVERWRITE ] = 0;
    else $setting[ SET_OVERWRITE ] = 1;

    // UDP送信
    if( isset( $_POST[ 'udp'.$settingno ] ) ) $setting[ SET_UDP ] = 1;
    else $setting[ SET_UDP ] = 0;
    if( isset( $_POST[ 'ip'.$settingno ] ) ) $setting[ SET_IP ] = trim( $_POST[ 'ip'.$settingno ] );
    if( isset( $_POST[ 'port'.$settingno ] ) ) $setting[ SET_PORT ] = (int) $_POST[ 'port'.$settingno ];

    // サスペンド
    if( isset( $_POST[ 'suspend'.$settingno ] ) ) $setting[ SET_SUSPEND ] = 1;
    else $setting[ SET_SUSPEND ] = 0;
}

    
// 録画設定を更新する
function update_setting( &$setting_array, $operate, $maxnumber )
{
    $lock = new Lock( FILELOCKDIR );

    show_debug( 'update_setting' );

    $updated = FALSE;
    for( $settingno = 0; $settingno < $maxnumber; ++$settingno ){

        // 未設定の時はデフォルト値をセット
        if( count( $setting_array ) <= $settingno ){
            $setting_array[ $settingno ] = get_default_setting();
            $setting_array[ $settingno ][ SET_ID ] = time()+$settingno;
            $updated = TRUE;
        }

        $setting = &$setting_array[ $settingno ];

        // 設定が壊れていたらデフォルト値をセット
        if( ! is_array( $setting ) || count( $setting ) != SET_SIZE ){

            show_debug( $settingno.' is broken' );

            $setting = get_default_setting();
            $setting[ SET_ID ] = time()+$settingno;
            $updated = TRUE;
        }

        // POST された値を入力
        if( $operate == OP_REC || $operate == OP_PLAY || $operate == OP_SAVE ){
            update_settingitem( $settingno, $setting );
            $updated = TRUE;
        }
    }

    if( $updated ) save_csv( get_settingfile(), $setting_array );
}


// 予約設定を更新する
function update_reserv( &$setting_array, $opnumber, $id )
{
    $lock = new Lock( FILELOCKDIR );

    // $id から編集する設定の設定番号を取得
    $setting = FALSE;
    for( $settingno = 0; $settingno < count( $setting_array ); ++$settingno ){

        if( $setting_array[ $settingno ][ SET_ID ] == $id ){
            $setting = &$setting_array[ $settingno ];
            break;
        }
    }

    show_debug( 'update_reserv opnumber = '.$opnumber );
    show_debug( 'id = '.$id.' settingno = '.$settingno );

    // 設定が見付からなければ新規に追加
    if( ! $setting ){
         $setting_array[] = get_default_setting();
         $setting = &$setting_array[ $settingno ];
         $setting[ SET_ID ] = time();
    }

    update_settingitem( $opnumber, $setting );

    $reservfile = get_reservfile();
    if( count( $setting_array ) ) save_csv( $reservfile, $setting_array );
    else unlink( $reservfile );
}


// 予約設定を削除する
function remove_reserv( &$setting_array, $id )
{
    $lock = new Lock( FILELOCKDIR );

    $setting = FALSE;
    for( $settingno = 0; $settingno < count( $setting_array ); ++$settingno ){

        if( $setting_array[ $settingno ][ SET_ID ] == $id ){
            $setting = &$setting_array[ $settingno ];
            break;
        }
    }

    show_debug( 'remove_reserv' );
    show_debug( 'id = '.$id.' settingno = '.$settingno );

    if( $setting ) array_splice( $setting_array, $settingno, 1 );

    $reservfile = get_reservfile();
    if( count( $setting_array ) ) save_csv( $reservfile, $setting_array );
    else unlink( $reservfile );
}


// 動いていない recfriio の録画情報を消す
function remove_status( &$status_array )
{
    $lock = new Lock( FILELOCKDIR );

    $removed = FALSE;
    $flag = TRUE;

    while( $flag ){

        $flag = FALSE;
        for( $i = 0; $i < count( $status_array ); ++$i ){

            if( ! is_running( $status_array[ $i ][ PID ] ) ){

                array_splice( $status_array, $i, 1 );

                $flag = TRUE;
                $removed = TRUE;
                break;
            }
        }
    }

    if( $removed ){
        $statusfile = get_statusfile();
        if( ! count( $status_array ) ) unlink( $statusfile );
        else save_csv( $statusfile, $status_array );
    }
}


// POST 時に操作を取得
// 戻り値 操作名
// $opnumber に番号が入る
function get_operate( &$opnumber, $maxnumber )
{
    $operate = OP_NONE;
    $opnumber = -1;

    if( $_SERVER[ "REQUEST_METHOD" ] == 'POST' ){

        if( isset( $_POST[ 'save' ] ) ) $operate = OP_SAVE;

        else for( $i = 0; $i < $maxnumber ; ++$i ){

                if( isset( $_POST[ 'play'.$i ] ) ){
                    $operate = OP_PLAY;
                    $opnumber = $i;
                    break;
                }
                else if( isset( $_POST[ 'rec'.$i  ] ) ){
                    $operate = OP_REC;
                    $opnumber = $i;
                    break;
                }
                else if( isset( $_POST[ 'stop'.$i  ] ) ){
                    $operate = OP_STOP;
                    $opnumber = $i;
                    break;
                }
                else if( isset( $_POST[ 'update'.$i  ] ) ){
                    $operate = OP_UPDATE;
                    $opnumber = $i;
                    break;
                }
                else if( isset( $_POST[ 'edit'.$i  ] ) ){
                    $operate = OP_EDIT;
                    $opnumber = $i;
                    break;
                }
                else if( isset( $_POST[ 'del'.$i  ] ) ){
                    $operate = OP_DEL;
                    $opnumber = $i;
                    break;
                }
            }
    }

    show_debug( 'get_operate : operate = '.$operate.' opnumber = '.$opnumber );
    return $operate;
}


// csvファイル読み込み
function read_csv( $file, &$data )
{
    show_debug( 'read_csv : '.$file );

    $data = array();

    if( ! file_exists( $file ) ){
        throw new Exception( 'エラー(read_csv) :ファイル('.$file.')が存在しません。' );
    }

    if( ! ( $fp = fopen( $file, 'r') ) ){
        throw new Exception( 'エラー(read_csv) :ファイル('.$file.')のオープンに失敗しました。パーミッションを確認してください。' );
    }
    while( $tmp = fgetcsv( $fp ) ) $data[] = $tmp;
    fclose( $fp );

    return TRUE;
}


// csvファイル書き込み
function save_csv( $file, &$data )
{
    show_debug( 'save_csv : '.$file );

    if( ! is_array( $data ) ) return FALSE;

    if( ! ( $fp = fopen( $file, 'w+') ) ){
        throw new Exception( 'エラー(save_csv) :ファイル('.$file.')のオープンに失敗しました。パーミッションを確認してください。' );
    }

    foreach( $data as $line ) fputcsv( $fp, $line );
    fclose( $fp );

    return TRUE;
}


// プロセスが動いているか
function is_running( $pid )
{
    if( $pid ){

        show_debug( 'is_running : pid = '.$pid );

        $ret = exec( 'ps --no-headers -p '.$pid );
        show_debug( $ret );

        if( $ret ) return TRUE;
    }

    return FALSE;
}


// ログ保存
function write_log( $log )
{
    $log = date( 'y/m/d H:i:s' ,time() ).' : '.$log."\n";

    if( ( $fp = fopen( get_vardir().LOGFILE, 'a' ) ) ) {
        fputs( $fp, $log  );
        fclose( $fp );
    }
    else throw new Exception( 'エラー(write_log): ログファイルのオープンに失敗しました。' );
}


// 予約設定から次の録画日時が正しいか調べる
function check_date( &$setting )
{
    $time = time();
    $year = (int)date( 'Y', $time );
    if( mktime( $setting[ SET_HOUR ], $setting[ SET_MIN ], 0, $setting[ SET_MONTH ], $setting[ SET_DAY ], $year ) < $time ) ++$year;

    show_debug( 'check_date : '.$year.'/'.$setting[ SET_MONTH ].'/'.$setting[ SET_DAY ] );

    if( $setting[ SET_MONTH ] <= 0 || $setting[ SET_MONTH ] > 12  || $setting[ SET_DAY ] <= 0 
        || ! checkdate( $setting[ SET_MONTH ], $setting[ SET_DAY ], $year ) ) return FALSE;

    return TRUE;
}


// 予約設定から次の録画日時を求める
// 戻り値は UNIX 時刻
function get_nexttime( &$setting, $time )
{
    $year = (int)date( 'Y', $time );

    // 日付指定
    if( $setting[ SET_MODE ] == MODE_ONCE ){

           $nexttime = mktime( $setting[ SET_HOUR ], $setting[ SET_MIN ], 0, $setting[ SET_MONTH ], $setting[ SET_DAY ], $year );

           // 来年か ?
           if( $nexttime < $time ) $nexttime = mktime( $setting[ SET_HOUR ], $setting[ SET_MIN ], 0, $setting[ SET_MONTH ], $setting[ SET_DAY ], $year+1 );
    }

    // 曜日指定
    else{

        $month = (int)date( 'm', $time );
        $day = (int)date( 'd', $time );
        $hour = (int)date( 'H', $time );
        $min = (int)date( 'i', $time );
        $week = (int)date( 'w', $time );

        $nexttime = mktime( $setting[ SET_HOUR ], $setting[ SET_MIN ], 0, $month, $day, $year );

        // 今日
        if( $setting[ SET_SUN + $week ] 
            && ( $setting[ SET_HOUR ] > $hour || ( $setting[ SET_HOUR ] == $hour && $setting[ SET_MIN ] > $min ) ) ){}

        // 明日以降
        else{

            for( $i = 1; $i < 8; ++$i ){
    
                $nexttime += ( 3600 * 24 );

                if( $setting[ SET_SUN + ( $week + $i ) % 7 ] ) break;
            }
        }
    }

    return $nexttime;
}


// 予約設定の配列から次の録画日時を求める
// 戻り値は UNIX 時刻
function get_nexttime_array( &$setting_array )
{
    $time = time();
    $nexttime = 0;

    foreach( $setting_array as $setting ){

        if( $setting[ SET_ACTIVE ] ){

            $nexttime_tmp = get_nexttime( $setting, $time );
            if( ! $nexttime || $nexttime_tmp < $nexttime ) $nexttime = $nexttime_tmp;
        }
    }

    show_debug( 'get_nexttime_array = '.date( 'Y/m/d H:i:D', $nexttime ) );

    return $nexttime;
}


// 現在録画中か
function is_recording( &$setting, $time )
{
    $duration = $setting[ SET_DURATION ];
    $sleepsec = $setting[ SET_SLEEPSEC ];
    if( $sleepsec ) $duration += ( 60 - $sleepsec );

    $nexttime = get_nexttime( $setting, $time - $duration );
    if( $nexttime <= $time && $time <= $nexttime + $duration ){

        $statusfile = get_statusfile();
        $status_array = array();
        if( file_exists( $statusfile ) ){

            read_csv( $statusfile, $status_array );

            foreach( $status_array as $status ) if( $status[ ID ] == $setting[ SET_ID ] ) return TRUE;
        }
    }

    return FALSE;
}


?>
